package br.pucrs.ages.victims.auth.config

import br.pucrs.ages.victims.auth.config.properties.KeycloakProperties
import org.keycloak.OAuth2Constants
import org.keycloak.admin.client.Keycloak
import org.keycloak.admin.client.KeycloakBuilder
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.web.reactive.function.client.WebClient


@Configuration
class KeycloakConfiguration(
    @Autowired val keycloakProperties: KeycloakProperties
) {

    @Bean
    fun keycloakWebClient() = WebClient.builder()
        .baseUrl("${keycloakProperties.url}/realms/${keycloakProperties.realm}")
        .build()


    @Bean
    fun keycloak(): Keycloak {
        return KeycloakBuilder.builder()
            .serverUrl(keycloakProperties.url)
            .realm(keycloakProperties.realm)
            .grantType(OAuth2Constants.PASSWORD)
            .clientId(keycloakProperties.clientId)
            .clientSecret(keycloakProperties.clientSecret)
            .username(keycloakProperties.username)
            .password(keycloakProperties.password)
            .build()
    }
}