package br.pucrs.ages.victims.auth.config.security

import br.pucrs.ages.victims.auth.config.security.jwt.TokenConverter
import org.springframework.beans.factory.annotation.Qualifier
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.security.config.annotation.method.configuration.EnableMethodSecurity
import org.springframework.security.config.annotation.web.builders.HttpSecurity
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity
import org.springframework.security.config.core.GrantedAuthorityDefaults
import org.springframework.security.web.SecurityFilterChain
import org.springframework.web.cors.CorsConfiguration
import org.springframework.web.cors.CorsConfigurationSource
import org.springframework.web.cors.UrlBasedCorsConfigurationSource


@Configuration
@EnableWebSecurity
@EnableMethodSecurity(prePostEnabled = true)
class SecurityConfig {

    @Bean
    fun filterChain(
        httpSecurity: HttpSecurity,
        @Qualifier("customAuthenticationEntryPoint") authEntryPoint: RestAuthenticationEntryPoint
    ): SecurityFilterChain {
        httpSecurity.authorizeHttpRequests { authorizeRequests ->
            authorizeRequests
                .requestMatchers("/users").permitAll()
                .requestMatchers("/users/admin").permitAll()
                .requestMatchers("/login").permitAll()
                .requestMatchers("/swagger-ui.html").permitAll()
                .requestMatchers("/swagger-ui/**").permitAll()
                .requestMatchers("/v3/api-docs/**").permitAll()
                .requestMatchers("/actuator/**").permitAll()
                .anyRequest().authenticated()
        }
        httpSecurity.exceptionHandling {
            it.authenticationEntryPoint(authEntryPoint)
        }

        httpSecurity.httpBasic {
            it.disable()
        }
        httpSecurity.cors {
            it.configurationSource(corsConfigurationSource())
        }
        httpSecurity.csrf {
            it.disable()
        }
        httpSecurity.oauth2ResourceServer { server ->
            server.authenticationEntryPoint(authEntryPoint)
            server.jwt {
                it.jwtAuthenticationConverter(TokenConverter())
            }
        }

        return httpSecurity.build()
    }

    @Bean
    fun corsConfigurationSource(): CorsConfigurationSource {
        val configuration = CorsConfiguration()
        configuration.addAllowedOrigin("*")
        configuration.addAllowedMethod("*")
        configuration.addAllowedHeader("*")
        val source = UrlBasedCorsConfigurationSource()
        source.registerCorsConfiguration("/**", configuration)
        return source
    }

    @Bean
    fun grantedAuthorityDefaults(): GrantedAuthorityDefaults = GrantedAuthorityDefaults("")
}