package br.pucrs.ages.victims.auth.config.security.principal

import org.springframework.security.oauth2.jwt.Jwt
import java.util.*

data class KeycloakPrincipal(
    val id: UUID,
    val email: String,
    val username: String,
    val roles: Set<String>,
) {
    companion object {
        fun fromJwt(jwt: Jwt): KeycloakPrincipal {
            fun loadClaim(key: String): String = jwt.claims[key].toString()

            return KeycloakPrincipal(
                id = UUID.fromString(loadClaim("app_id")),
                email = loadClaim("email"),
                username = loadClaim("preferred_username"),
                roles = getRoles(jwt)
            )
        }

        private fun getRoles(jwt: Jwt): Set<String> {
            val realmAccess: Map<String, Collection<String>> = jwt.getClaim("realm_access")
            return realmAccess["roles"]!!.toSet()
        }
    }

}
