package br.pucrs.ages.victims.auth.mapper

import br.pucrs.ages.victims.auth.domain.User
import br.pucrs.ages.victims.auth.domain.dto.UserDto
import br.pucrs.ages.victims.auth.repository.keycloak.APPLICATION_ID
import br.pucrs.ages.victims.auth.repository.keycloak.PHONE_NUMBER
import br.pucrs.ages.victims.auth.web.representation.response.UserResponse
import org.keycloak.representations.idm.CredentialRepresentation
import org.keycloak.representations.idm.UserRepresentation
import org.springframework.stereotype.Component
import java.util.*

@Component
class UserMapper {

    fun fromRepresentationToDomain(representation: UserRepresentation): User {
        return User(
            id = UUID.fromString(representation.attributes[APPLICATION_ID]?.firstOrNull() ?: ""),
            name = representation.firstName,
            email = representation.email,
            phoneNumber = representation.attributes[PHONE_NUMBER]?.firstOrNull() ?: "",
            cpf = representation.username,
            active = representation.isEnabled
        )
    }

    fun toRepresentation(user: User): UserRepresentation {
        val userRepresentation = UserRepresentation()

        userRepresentation.username = user.cpf
        userRepresentation.email = user.email
        userRepresentation.firstName = user.name
        userRepresentation.isEnabled = true
        userRepresentation.isEmailVerified = true

        userRepresentation.attributes = mapOf(
            APPLICATION_ID to listOf(user.id.toString()),
            PHONE_NUMBER to listOf(user.phoneNumber)
        )

        return userRepresentation
    }

    fun mapPasswordToKeycloakCredential(
        password: String,
        isTemporary: Boolean
    ): CredentialRepresentation {
        val credential = CredentialRepresentation()
        credential.type = CredentialRepresentation.PASSWORD
        credential.value = password
        credential.isTemporary = isTemporary
        return credential
    }

    fun toResponse(userDTO: UserDto) = UserResponse(
        id = userDTO.id,
        name = userDTO.name,
        email = userDTO.email,
        cpf = userDTO.cpf,
        phoneNumber = userDTO.phoneNumber,
        active = userDTO.active,
        roles = userDTO.roles
    )

}
