package br.pucrs.ages.victims.auth.repository.keycloak

import br.pucrs.ages.victims.auth.config.properties.KeycloakProperties
import br.pucrs.ages.victims.auth.domain.dto.JwtTokenDTO
import br.pucrs.ages.victims.auth.exception.AuthenticationException
import br.pucrs.ages.victims.auth.repository.AuthRepository
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Repository
import org.springframework.web.reactive.function.BodyInserters
import org.springframework.web.reactive.function.client.WebClient

@Repository
class AuthRepositoryImpl(
    @Autowired
    private val webClient: WebClient,
    @Autowired
    private val keycloakProperties: KeycloakProperties
) : AuthRepository {

    override fun authenticate(username: String, password: String): JwtTokenDTO {
        return webClient.post()
            .uri("/protocol/openid-connect/token")
            .body(
                BodyInserters.fromFormData("client_id", keycloakProperties.clientId)
                    .with("client_secret", keycloakProperties.clientSecret)
                    .with("username", username)
                    .with("password", password)
                    .with("grant_type", "password")
            )
            .retrieve()
            .bodyToMono(JwtTokenDTO::class.java)
            .block() ?: throw AuthenticationException("Invalid credentials")
    }
}