package br.pucrs.ages.victims.auth.validator

import br.pucrs.ages.victims.auth.exception.ValidationException
import br.pucrs.ages.victims.auth.web.representation.request.RegisterAdminRequest
import br.pucrs.ages.victims.auth.web.representation.request.RegisterUserRequest
import org.springframework.stereotype.Component

const val MIN_PASSWORD_SIZE: Int = 8

@Component
class UserValidator {

    fun validatePayload(payload: RegisterUserRequest) {
        if (!isPasswordValid(payload.password)) {
            throw ValidationException("Senha inválida")
        }

        if (!payload.phoneNumber.all { it.isDigit() }) {
            throw ValidationException("O telefone deve conter apenas números")
        }
    }

    fun validateAdminPayload(payload: RegisterAdminRequest) {
        if (!isPasswordValid(payload.password)) {
            throw ValidationException("Senha inválida")
        }

        if (!payload.inviteId.matches(Regex("[a-f0-9]{8}(-[a-f0-9]{4}){3}-[a-f0-9]{12}"))) {
            throw ValidationException("O convite informado não é válido")
        }
    }


    private fun isPasswordValid(password: String): Boolean {
        return password.length >= MIN_PASSWORD_SIZE &&
                password.firstOrNull { it.isDigit() } != null &&
                password.filter { it.isLetter() }.firstOrNull { it.isUpperCase() } != null &&
                password.filter { it.isLetter() }.firstOrNull { it.isLowerCase() } != null
    }
}