package br.pucrs.ages.victims.auth.web

import br.pucrs.ages.victims.auth.domain.User
import br.pucrs.ages.victims.auth.domain.dto.Page
import br.pucrs.ages.victims.auth.web.representation.request.RegisterAdminRequest
import br.pucrs.ages.victims.auth.web.representation.request.RegisterUserRequest
import br.pucrs.ages.victims.auth.web.representation.response.UserResponse
import io.swagger.v3.oas.annotations.Operation
import io.swagger.v3.oas.annotations.responses.ApiResponse
import io.swagger.v3.oas.annotations.responses.ApiResponses
import io.swagger.v3.oas.annotations.security.SecurityRequirement
import io.swagger.v3.oas.annotations.tags.Tag
import org.springframework.http.ResponseEntity
import java.util.UUID

@Tag(name = "Usuário", description = "Endpoints de usuário")
interface UserApi {

    @Operation(summary = "Retorna o usuário logado")
    @ApiResponses(
        value = [
            ApiResponse(responseCode = "200", description = "Usuário encontrado com sucesso"),
            ApiResponse(responseCode = "401", description = "Usuário não autenticado"),
            ApiResponse(responseCode = "500", description = "Erro interno")
        ]
    )
    fun getCurrentUser(): ResponseEntity<UserResponse>

    @Operation(summary = "Retorna o usuário pelo id")
    @ApiResponses(
        value = [
            ApiResponse(responseCode = "200", description = "Usuário encontrado com sucesso"),
            ApiResponse(responseCode = "401", description = "Usuário não autenticado"),
            ApiResponse(responseCode = "403", description = "Usuário não tem permissão para buscar usuários"),
            ApiResponse(responseCode = "500", description = "Erro interno")
        ]
    )
    fun findById(id: UUID): ResponseEntity<User>

    @Operation(summary = "Cria um usuário no keycloak e retorna o id usado pela aplicação")
    @ApiResponses(
        value = [
            ApiResponse(responseCode = "200", description = "Usuário criado com sucesso"),
            ApiResponse(responseCode = "400", description = "Payload inválido"),
            ApiResponse(responseCode = "500", description = "Erro interno")
        ]
    )
    fun create(payload: RegisterUserRequest): ResponseEntity<User>

    @Operation(summary = "Buscar os usuários cadastrados no keycloak")
    @ApiResponses(
        value = [
            ApiResponse(responseCode = "200", description = "Usuários encontrados com sucesso"),
            ApiResponse(responseCode = "401", description = "Usuário não autenticado"),
            ApiResponse(responseCode = "403", description = "Usuário não tem permissão para buscar usuários"),
            ApiResponse(responseCode = "500", description = "Erro interno")
        ]
    )
    @SecurityRequirement(name = "bearer token")
    fun findAll(nameFilter: String?, active: Boolean?, pageNumber: Int?, pageSize: Int?): ResponseEntity<Page<User>>

    @Operation(summary = "Deleta um usuário do keycloak (logicamente)")
    @ApiResponses(
        value = [
            ApiResponse(responseCode = "200", description = "Usuário deletado com sucesso"),
            ApiResponse(responseCode = "400", description = "Payload inválido"),
            ApiResponse(responseCode = "401", description = "Usuário não autenticado"),
            ApiResponse(responseCode = "403", description = "Usuário não tem permissão para deletar usuários"),
            ApiResponse(responseCode = "500", description = "Erro interno")
        ]
    )
    @SecurityRequirement(name = "bearer token")
    fun delete(id: UUID): ResponseEntity<Unit>

    @Operation(summary = "Cria um usuário admin no keycloak e retorna o id usado pela aplicação")
    @ApiResponses(
        value = [
            ApiResponse(responseCode = "200", description = "Usuário criado com sucesso"),
            ApiResponse(responseCode = "400", description = "Payload inválido"),
            ApiResponse(responseCode = "500", description = "Erro interno")
        ]
    )
    fun createAdmin(payload: RegisterAdminRequest): ResponseEntity<User>
}