package br.pucrs.ages.victims.auth.web.controller

import br.pucrs.ages.victims.auth.domain.User
import br.pucrs.ages.victims.auth.domain.dto.Page
import br.pucrs.ages.victims.auth.mapper.UserMapper
import br.pucrs.ages.victims.auth.service.UserService
import br.pucrs.ages.victims.auth.web.UserApi
import br.pucrs.ages.victims.auth.web.representation.request.RegisterAdminRequest
import br.pucrs.ages.victims.auth.web.representation.request.RegisterUserRequest
import br.pucrs.ages.victims.auth.web.representation.response.UserResponse
import jakarta.validation.Valid
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.*
import java.util.*

@RestController
@RequestMapping("/users")
class UserController(
    @Autowired
    private val userService: UserService,
    @Autowired
    private val userMapper: UserMapper
) : UserApi {

    @GetMapping
    @PreAuthorize("hasRole('ADMIN')")
    override fun findAll(
        @RequestParam(required = false) nameFilter: String?,
        @RequestParam(required = false) active: Boolean?,
        @RequestParam(required = false) pageNumber: Int?,
        @RequestParam(required = false) pageSize: Int?,
    ): ResponseEntity<Page<User>> {
        return ResponseEntity.ok(
            userService.findAll(nameFilter, active, pageNumber, pageSize)
        )
    }

    @GetMapping("/me")
    override fun getCurrentUser(): ResponseEntity<UserResponse> {
        val userDTO = userService.getCurrentUser()
        return ResponseEntity.ok(userMapper.toResponse(userDTO))
    }

    @GetMapping("/{id}")
    @PreAuthorize("hasRole('ADMIN')")
    override fun findById(@PathVariable id: UUID): ResponseEntity<User> {
        val user = userService.findById(id)
        return ResponseEntity.ok(user)
    }

    @PostMapping
    override fun create(@Valid @RequestBody payload: RegisterUserRequest): ResponseEntity<User> {
        val user = userService.create(payload)
        return ResponseEntity.status(HttpStatus.CREATED).body(user)
    }

    @PostMapping("/admin")
    override fun createAdmin(@Valid @RequestBody payload: RegisterAdminRequest): ResponseEntity<User> {
        val user = userService.createAdmin(payload)
        return ResponseEntity.status(HttpStatus.CREATED).body(user)
    }

    @PreAuthorize("hasRole('ADMIN')")
    @DeleteMapping("/{id}")
    override fun delete(@PathVariable id: UUID): ResponseEntity<Unit> {
        userService.delete(id)
        return ResponseEntity.noContent().build()
    }

}