package br.pucrs.ages.victims.auth.service

import br.pucrs.ages.victims.auth.domain.dto.JwtTokenDTO
import br.pucrs.ages.victims.auth.exception.AuthenticationException
import br.pucrs.ages.victims.auth.repository.AuthRepository
import io.mockk.every
import io.mockk.mockk
import io.mockk.verify
import org.junit.jupiter.api.Assertions.assertNotNull
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.assertThrows


class AuthServiceTest {

    private val authRepository = mockk<AuthRepository>()

    private val authService = AuthService(authRepository)

    @Test
    fun `should authenticate when no error happens in authentication`() {
        // Arrange
        every { authRepository.authenticate(any(), any()) } returns aJwtTokenDTO()

        val username = "username"
        val password = "password"

        // Act
        val jwtTokenDTO = authService.login(username, password)

        // Assert
        verify { authRepository.authenticate(username, password) }
        assertNotNull(jwtTokenDTO)
    }

    @Test
    fun `should not authenticate when an error in thrown`() {
        // Arrange
        every { authRepository.authenticate(any(), any()) } throws Exception()

        val username = "username"
        val password = "password"

        // Act
        assertThrows<AuthenticationException>("CPF ou senha inválidos") { authService.login(username, password) }

        // Assert
        verify { authRepository.authenticate(username, password) }
    }


    private fun aJwtTokenDTO() = JwtTokenDTO(
        accessToken = "accessToken",
        refreshToken = "refreshToken",
        refreshTokenExpiresIn = 0L,
        tokenType = "bearer",
        expiresIn = 0L
    )
}