package br.pucrs.ages.victims.auth.validator

import br.pucrs.ages.victims.auth.exception.ValidationException
import br.pucrs.ages.victims.auth.web.representation.request.RegisterUserRequest
import org.junit.jupiter.api.Assertions.*
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.assertThrows

class UserValidatorTest {

    private val userValidator = UserValidator()

    @Test
    fun `should throw error when password is less than 8 characters`() {
        val payload = RegisterUserRequest(
            password = "1234567"
        )

        assertThrows<ValidationException> { userValidator.validatePayload(payload)}
    }

    @Test
    fun `should throw error when password does not contain a digit`() {
        val payload = RegisterUserRequest(
            password = "Abc/**/"
        )

        assertThrows<ValidationException> { userValidator.validatePayload(payload)}
    }

    @Test
    fun `should throw error when password does not contain an uppercase letter`() {
        val payload = RegisterUserRequest(
            password = "abc123/**/"
        )

        assertThrows<ValidationException> { userValidator.validatePayload(payload)}
    }

    @Test
    fun `should throw error when password does not contain a lowercase letter`() {
        val payload = RegisterUserRequest(
            password = "ABC123/**/"
        )

        assertThrows<ValidationException> { userValidator.validatePayload(payload)}
    }

    @Test
    fun `should throw error when phone number is invalid`() {
        val payload = RegisterUserRequest(
            password = "Abc123/**/",
            phoneNumber = "AAAAAAAA"
        )

        assertThrows<ValidationException> { userValidator.validatePayload(payload)}
    }

    @Test
    fun `should not throw error when payload is valid`() {
        val payload = RegisterUserRequest(
            password = "Abc123/**/",
            phoneNumber = "51999999999"
        )

        assertDoesNotThrow { userValidator.validatePayload(payload)}
    }
}